@@ -5,7 +5,7 @@ from copy import deepcopy
 from radish.utils import console_write
 from terraform_compliance.common.defaults import Defaults
 from terraform_compliance.extensions.cache import Cache
-from terraform_compliance.common.exceptions import TerraformComplianceInternalFailure
+from terraform_compliance.common.helper import recursive_jsonify
 
 
 class TerraformParser(object):
@@ -23,9 +23,14 @@ class TerraformParser(object):
         self.supported_terraform_versions = (
             '0.12.',
             '0.13.',
-            '0.14.'
+            '0.14.',
+            '0.15.',
+            '1.0.',
         )
-        self.supported_format_versions = ['0.1']
+        self.supported_format_versions = [
+            '0.1',
+            '0.2'
+        ]
 
         self.raw = self._read_file(filename)
         self.variables = None
@@ -156,7 +161,7 @@ class TerraformParser(object):
         Note: This function may be extended to capture 'after' values as well. That would require flattening the multi level
         dictionaries in resource
         '''
-        
+
         # get type
         resource_type = resource.get('type', '')
 
@@ -203,8 +208,7 @@ class TerraformParser(object):
         resources = self.raw.get('configuration', {}).get('root_module', {}).get('resources', [])
 
         # Append module resources
-        for module in seek_key_in_dict(self.raw.get('configuration', {}).get('root_module', {}).get("module_calls", {}), "module"):
-            resources += module.get('module',{}).get("resources", [])
+        resources.extend(self.process_module_calls(self.raw.get('configuration', {}).get('root_module', {}).get("module_calls", {})))
 
         remove_constant_values(resources)
         for resource in resources:
@@ -361,7 +365,7 @@ class TerraformParser(object):
                     for ref in valid_references:
                         # if ref is not in the correct format, handle it
                         if len(ref.split('.')) < 3 and ref.startswith('module'):
-                            
+
                             # Using for_each and modules together may introduce an issue where the plan.out.json won't include the necessary third part of the reference
                             # It is partially resolved by mounting the reference to all instances belonging to the module
                             if 'for_each_expression' in self.configuration['resources'][resource]:
@@ -392,9 +396,9 @@ class TerraformParser(object):
                                         ' It will be mounted to the following resources:').format(ref, resource)))
                                 for i, r in enumerate(ambigious_references, 1):
                                     console_write(defaults.info_colour('{}. {}'.format(i, r)))
-                                
+
                             # if the reference can not be resolved, warn the user and continue.
-                            else: 
+                            else:
                                 console_write('{} {}: {}'.format(Defaults().warning_icon,
                                        Defaults().warning_colour('WARNING (Mounting)'),
                                        Defaults().info_colour('The reference "{}" in resource {} is ambigious. It will not be mounted.'.format(ref, resource))))
@@ -423,8 +427,7 @@ class TerraformParser(object):
                         ref_type = resource.split('.')[0]
 
                     if not ref_type and not self.is_type(resource, 'data'):
-                        resource_type, resource_id = resource.split('.')
-                        ref_type = resource_type
+                        ref_type = self.extract_resource_type_from_address(resource)
 
                     for k, v in ref_list.items():
                         v = flatten_list(v)
@@ -434,12 +437,12 @@ class TerraformParser(object):
                     self._mount_resources(source=source_resources,
                                           target=ref_list,
                                           ref_type=ref_type)
-                    
+
                     # Mounting B->A
                     for parameter, target_resources in ref_list.items():
                         for target_resource in target_resources:
                             if not self.is_type(resource, 'data') and not self.is_type(resource, 'var') and not self.is_type(resource, 'provider'):
-                                ref_type = target_resource.split('.', maxsplit=1)[0]
+                                ref_type = self.extract_resource_type_from_address(target_resource)
 
                                 self._mount_resources(source=[target_resource],
                                                       target={parameter: source_resources},
@@ -482,6 +485,13 @@ class TerraformParser(object):
             self._mount_references()
             self._add_action_status()
 
+            self.resources = recursive_jsonify(self.resources)
+            self.resources_raw = recursive_jsonify(self.resources_raw)
+            self.type_to_after_unknown_properties = recursive_jsonify(self.type_to_after_unknown_properties)
+            self.variables = recursive_jsonify(self.variables)
+            self.data = recursive_jsonify(self.data)
+            self.providers = recursive_jsonify(self.providers)
+
             if self.parse_it:
                 self.cache.set('mounted_resources', self.resources)
                 self.cache.set('resources_raw', self.resources_raw)
@@ -564,3 +574,61 @@ class TerraformParser(object):
             return resource['address'].split('.')[0] == mode
 
         return False
+
+    def process_module_calls(self, module_resource, parents_modules=None):
+        '''
+        This method will recursively process modules and extract resources from "module_resource" data
+        which is actually a data from self.configuration dict. We were returning the native resource name
+        before this method, but now we are returning proper address naming for the resource.
+
+        :param module_resource: The self.configuration part
+        :param parents_modules: internal usage for recursive functionality
+        :return: None
+        '''
+        if parents_modules is None:
+            parents_modules = []
+
+        resources = []
+        for k, v in module_resource.items():
+            # Set the naming correct (for cases like module.a.module.b.module.c...)
+            current_module_level = deepcopy(parents_modules)
+            current_module_level.append('module.{}'.format(k))
+            module_name = ".".join(current_module_level)
+
+            # Register the resource (along with module naming)
+            if 'resources' in v.get('module', {}):
+                for resource in v['module']['resources']:
+                    resource['address'] = '{}.{}'.format(module_name, resource['address'])
+                    resources.append(resource)
+
+            # Dive deeper, its not finished yet.
+            if 'module_calls' in v.get('module', {}):
+                resources.extend(self.process_module_calls(v['module']['module_calls'], current_module_level))
+
+        return resources
+
+    def extract_resource_type_from_address(self, resource_address_string):
+        '''
+        Tries to get the resource type from the resource address
+
+        :param resource_address_string: String of the whole resource address
+        :return: String of the resource type if found, otherwise will return full address
+
+        Example;
+
+            "aws_s3_bucket.test" will return "aws_s3_bucket"
+            "module_a.module_b.module_c.aws_s3_bucket.test" will return "aws_s3_bucket"
+            "something_else" will return "something_else"
+        '''
+        if '.' in resource_address_string:
+            octets = resource_address_string.split('.')
+            if len(octets) > 1:
+                # Return the type as we found it properly
+                return octets[-2]
+            else:
+                # Return the whole address
+                return octets[0]
+
+        # Returning the whole address
+        return resource_address_string
+
